program jcMute_v2
  ! Program to mute gathers
  ! Handles both inner and outer mutes
  !
  ! Input:
  !   1) history file
  !   2) mopt [0]: mute type
  !	   - 0: binary mute
  !	       - vmute [1450]: mute velocity (parameter)
  !	       - t0 [0]: when to begin the linear mute (parameter)
  !        - 1: sin2 tapered mute
  !            - v1 [1450]: slower velocity (parameter)
  !            - v2 [2900]: faster velocity (parameter)
  !            - s1 [1450]: higher slowness (parameter)
  !            - s2 [2900]: lower slowness (parameter)
  !            - t01 [0.15]: when to begin the slower velocity ramp (parameter)
  !            - t02 [0.]: when to begin the faster velocity ramp (parameter)
  !   3) outer [y]: whether to do an inner or outer mute (inner is anything faster, outer is anything slower)
  !
  ! Output:
  !   1) history file
  !
  ! Jason Chang
  ! February 2014


  !=====================================
  ! standard declarations
  !=====================================
  use sep
  implicit none


  !=====================================
  ! declare variable types
  !=====================================
  ! file parameters
  integer                                     :: n1,n2,n3,n4
  real                                        :: o1,o2,o3,d1,d2,d3

  ! mute parameters
  integer                                     :: mopt
  real                                        :: vmute,v1,v2,t0,t01,t02
  logical                                     :: outer

  ! program parameters
  integer                                     :: i1,i2,i3
  real                                        :: smute,s1,s2
  real                                        :: x,t,weight
  real,parameter                              :: pi=3.14159265
  real,allocatable,dimension (:,:)            :: dat
  

  !=====================================
  ! I/O parameters
  !=====================================
  call sep_init()

  ! history file parameters
  call from_history('n1',n1,1 )
  call from_history('d1',d1,1.)
  call from_history('o1',o1,0.)
  call from_history('n2',n2,1 )
  call from_history('d2',d2,1.)
  call from_history('o2',o2,0.)
  call from_history('n3',n3,1 )
  call from_history('d3',d3,1.)
  call from_history('o3',o3,0.)
  call from_history('n4',n4,1 )

  ! mute paramters
  call from_param('mopt',mopt,0)
  call from_param('outer',outer,.true.)
  call from_param('vmute',vmute,1450.)
  call from_param('smute',smute,1./vmute)
  call from_param('t0',t0,0.)
  call from_param('v1',v1,1450.)
  call from_param('s1',s1,1./v1)
  call from_param('v2',v2,2900.)
  call from_param('s1',s2,1./v2)
  call from_param('t01',t01,0.15)
  call from_param('t02',t02,0.)
  if (t02.gt.t01) call erexit('t01 must be greater than t02')

  call sep_close()


  !=====================================
  ! allocate
  !=====================================
  allocate(dat(n1,n2))


  !=====================================
  ! begin kernel
  !=====================================
  ! loop over gathers
  do i3=1,n3*n4

    ! read in data
    dat = 0.
    call sreed('in',dat,4*n1*n2)

    ! loop over data
    do i2=1,n2

      ! set the x index
      x = o2+(i2-1)*d2

      do i1=1,n1

        ! set the t index
        t = o1+(i1-1)*d1

        !=====sin2 mute=====!
        if (mopt.eq.1) then
          ! outer mute case
          if (outer) then
            if (t.lt.(t02+x*s2)) then
              weight = 0.
            else if (t.gt.(t01+x*s1)) then
              weight = 1.
            else
              weight = sin(0.5*pi*(t-t02-x*s2)/(t01-t02+x*s1-x*s2))**2
            end if
          else
            if (t.gt.(t01+x*s1)) then
              weight = 0.
            else if (t.lt.(t02+x*s2)) then
              weight = 1.
            else
              weight = 1-sin(0.5*pi*(t-t02-x*s2)/(t01-t02+x*s1-x*s2))**2
            end if
          end if

        !===== binary mute =====!
        else
          ! outer mute case
          if (outer) then
            if (t.lt.(t0+x*smute)) then
              weight = 0.
            else
              weight = 1.
            end if
          ! inner mute case
          else
            if (t.lt.(t0+x*smute)) then
              weight = 1.
            else
              weight = 0.
            end if
          end if
        end if

        ! apply weight to data
        dat(i1,i2) = dat(i1,i2)*weight

      end do
    end do


    ! write out data
    call srite('out',dat,4*n1*n2)

  end do
  


  !=====================================
  ! clean up
  !=====================================
  deallocate(dat)
  call exit(0)

end program
