module snr_mod
  ! Module containing snr filter
  !
  ! Jason Chang
  ! April 2014

  !==========================================================
  ! standard declarations
  !==========================================================
  implicit none

  !==========================================================
  ! declare module variables
  !==========================================================
  integer,private                        :: n1,tau_ind
  real,private                           :: thresh

  contains

  !==========================================================
  ! snr initialization initialization
  !==========================================================
  subroutine snr_init(n1_in,thresh_in,tau_ind_in)

    !===========================================
    ! declare variables
    !===========================================
    integer                      :: n1_in,tau_ind_in
    real                         :: thresh_in

    !===========================================
    ! set these variables for entire module
    !===========================================
    n1 = n1_in
    thresh = thresh_in
    tau_ind = tau_ind_in

  end subroutine


  !=============================================================
  ! close subroutine
  !=============================================================
  subroutine snr_close()
  end subroutine


  !=============================================================
  ! return logical variable for snr based on time of maximum amplitude
  !=============================================================
  function logical_snr_max(dat) result(vsnr)

    !=============================
    ! declare variables
    !=============================
    integer                        :: ii,counter
    integer                        :: maxamploc,wlo,whi
    real                           :: maxamp,noise,snr
    real,allocatable,dimension (:) :: dat
    logical                        :: vsnr

    !=============================
    ! reset values
    !=============================
    maxamp = 0.
    counter = 0
    noise = 0.
    snr = 0.

    !=============================
    ! find the index and value of the maximum absolute amplitude
    !=============================
    do ii=1,n1
      if (abs(dat(ii)) > maxamp) then
        maxamploc = ii
        maxamp = abs(dat(ii))
      end if
    end do
    
    !=============================
    ! define a window centered about maximum amplitude
    !=============================
    ! calculate the centered window around the maximum
    wlo = maxamploc-tau_ind
    whi = maxamploc+tau_ind

    !=============================
    ! check the boundary of the windows
    ! need to try different cases
    !=============================
    ! if window is too big, then do nothing
    if ((wlo .lt. 1) .and. (whi .gt. n1)) then
      call erexit('Window is too big--no noise region')
    ! if window meets lower boundary, then only take later than window
    elseif (wlo .lt. 1) then
      do ii=whi+1,n1
        noise = noise + dat(ii)**2
        counter = counter + 1
      end do
    ! if at higher boundary, then only take earlier than window
    elseif (whi .gt. n1) then
      do ii=1,wlo-1
        noise = noise + dat(ii)**2
        counter = counter + 1
      end do
    ! otherwise take both earlier than later windows
    else
      do ii=1,wlo-1
        noise = noise + dat(ii)**2
        counter = counter + 1
      end do
      do ii=whi+1,n1
        noise = noise + dat(ii)**2
        counter = counter + 1
      end do
    end if

    ! calculate RMS of the noise
    noise = sqrt(noise/counter)
    
    ! compare ratio of max amplitude to RMS noise amplitude
    snr = maxamp/noise
write(0,*) snr

    ! return proper result
    if (snr.lt.thresh) vsnr = .false.
    if (snr.ge.thresh) vsnr = .true.

  end function


  !=============================================================
  ! return snr based on time of maximum amplitude
  !=============================================================
  function calc_snr_max(dat) result(snr)

    !=============================
    ! declare variables
    !=============================
    integer                        :: ii,counter
    integer                        :: maxamploc,wlo,whi
    real                           :: maxamp,noise,snr
    real,allocatable,dimension (:) :: dat

    !=============================
    ! reset values
    !=============================
    maxamp = 0.
    counter = 0
    noise = 0.
    snr = 0.

    !=============================
    ! find the index and value of the maximum absolute amplitude
    !=============================
    do ii=1,n1
      if (abs(dat(ii)) > maxamp) then
        maxamploc = ii
        maxamp = abs(dat(ii))
      end if
    end do
    
    !=============================
    ! define a window centered about maximum amplitude
    !=============================
    ! calculate the centered window around the maximum
    wlo = maxamploc-tau_ind
    whi = maxamploc+tau_ind

    !=============================
    ! check the boundary of the windows
    ! need to try different cases
    !=============================
    ! if window is too big, then do nothing
    if ((wlo .lt. 1) .and. (whi .gt. n1)) then
      call erexit('Window is too big--no noise region')
    ! if window meets lower boundary, then only take later than window
    elseif (wlo .lt. 1) then
      do ii=whi+1,n1
        noise = noise + dat(ii)**2
        counter = counter + 1
      end do
    ! if at higher boundary, then only take earlier than window
    elseif (whi .gt. n1) then
      do ii=1,wlo-1
        noise = noise + dat(ii)**2
        counter = counter + 1
      end do
    ! otherwise take both earlier than later windows
    else
      do ii=1,wlo-1
        noise = noise + dat(ii)**2
        counter = counter + 1
      end do
      do ii=whi+1,n1
        noise = noise + dat(ii)**2
        counter = counter + 1
      end do
    end if

    ! calculate RMS of the noise
    noise = sqrt(noise/counter)
    
    ! compare ratio of max amplitude to RMS noise amplitude
    snr = maxamp/noise

  end function
  

end module
