#! /usr/bin/env python

################################################################################
# supergather_patch.py
################################################################################
# Python code for create supergathers from chunks of the Long Beach array
# 	- essentially runs a bunch of Seplib commands
#	- input file is the cat of all days
#
# Jason Chang
# February 2014
################################################################################


########################################
# import modules
########################################
import sys
import commands
import math


########################################
# useful definitions
########################################
SEPLIB = '/opt/SEP/SEP8.0/bin'
#run = True
run = False

########################################
# grab command line
########################################
if len(sys.argv) !=2:
  print "Expecting one argument: [patch size]"
  sys.exit(0)
size = int(sys.argv[1])

########################################
# set parameters
########################################

# set the patch spacing parameters in km
npatchx = 5
opatchx = 1294.25
dpatchx = 0.75
npatchy = 10
opatchy = 1226.25
dpatchy = 0.75


# set the patch dimensions (km)
lpatchx = size
lpatchy = size

# set length of radial offset bins (km)
lh = 0.025



########################################
# define commands
########################################

# command for converting the header from ft to km
def header(daily_file,header_file,run):
  header_com = "Headermath < %s key1=receiverx eqn1='receiver_x*0.0003048' type1=scalar_float key2=receivery eqn2='receiver_y*0.0003048' type2=scalar_float key3=sourcex eqn3='source_x*0.0003048' type3=scalar_float key4=sourcey eqn4='source_y*0.0003048' type4=scalar_float key5=absoff eqn5='@SQRT((0.0003048*source_x-0.0003048*receiver_x)*(0.0003048*source_x-0.0003048*receiver_x)+(0.0003048*source_y-0.0003048*receiver_y)*(0.0003048*source_y-0.0003048*receiver_y))' type5=scalar_float > %s" %(daily_file,header_file)
  print header_com
  if run:
    commands.getstatusoutput(header_com)


# command for windowing out appropriate stations for creating supergathers
def windowkey(header_file,window_file,x,y,dx,dy,run):
  # calculate parameters
  minkx = x-(dx/2.)
  maxkx = x+(dx/2.)
  minky = y-(dy/2.)
  maxky = y+(dy/2.)
  # set command
  windowkey_com = "Window_key < %s key1=receiverx mink1=%s maxk1=%s key2=receivery mink2=%s maxk2=%s key3=sourcex mink3=%s maxk3=%s key4=sourcey mink4=%s maxk4=%s > %s" %(header_file,minkx,maxkx,minky,maxky,minkx,maxkx,minky,maxky,window_file)
  print windowkey_com
  if run:
    commands.getstatusoutput(windowkey_com)


# command for sorting patches by absolute offset and azimuth
def sort(window_file,sort_file,dx,dy,dh,run):
  # calculate parameters
  zz = (dx**2) + (dy**2)
  maxlength = math.sqrt(zz)
  nh = int(math.ceil(maxlength/dh))
  # set command
  sort_com = "%s/Sort3d < %s verb=1 nkeys=2 key1=azi ng1=181 dg1=0.01745 og1=-1.571 key2=absoff ng2=%s dg2=%s og2=0 compress_tr=0 > %s" %(SEPLIB,window_file,nh,dh,sort_file)
  print sort_com
  if run:
    commands.getstatusoutput(sort_com)


# command for stacking over the receivers
def stack(sort_file,stack_file,run):
  stack_com = "%s/Stack3d < %s normalize=1 maxsize=10000 | %s/Stack3d normalize=1 maxsize=10000 > %s" %(SEPLIB,sort_file,SEPLIB,stack_file)
  print stack_com
  if run:
    commands.getstatusoutput(stack_com)


# command for filling everything with zeros
def fillzeros(outfile,nt,ot,dt,dx,dy,dh,run):
  # calculate parameters
  zz = (dx**2) + (dy**2)
  maxlength = math.sqrt(zz)
  nh = int(math.ceil(maxlength/dh))
  # pretend this is a velocity file
  vel_com = 'Vel n1=%s n2=%s n3=1 d1=%s d2=%s d3=1 o1=%s o2=0 o3=1 vc=0. > %s' %(nt,nh,dt,dh,ot,outfile)
  print vel_com
  if run:
    commands.getstatusoutput(vel_com)
  # echo in the proper axes parameters
  echo_com = """echo "n4=1 o4=1 d4=1 n5=1 o5=1 d5=1" >> %s""" %(outfile)
  print echo_com
  if run:
    commands.getstatusoutput(echo_com)


# command for catting along x-direction
def catx(stackfiles,catxfile,run):
  # setup stackfiles
  stacks = ' '.join(stackfiles)
  catx_com = "Cat3d axis=3 %s > %s" %(stacks,catxfile)
  print catx_com
  if run:
    commands.getstatusoutput(catx_com)


# command for catting along y-direction
def caty(catfiles,outfile,run):
  # setup stackfiles
  cats = ' '.join(catfiles)
  caty_com = "Cat3d axis=4 %s > %s" %(cats,outfile)
  print caty_com
  if run:
    commands.getstatusoutput(caty_com)


# command for echoing in the correct header information
def echo(filename,run):
  echo_com = """echo "label1='time lag (s)' label2='abs_off (km)' label3='x location (km)' label4='y location (km)' o3=%s d3=%s o4=%s d4=%s" >> %s""" %(opatchx,dpatchx,opatchy,dpatchy,filename)
  print echo_com
  if run:
    commands.getstatusoutput(echo_com)


#########################################
# run commands
#########################################
# create lists of x and y centered coordinates
listx = []
listy = []
for ix in range(npatchx):
  xx = opatchx + (ix*dpatchx)
  listx.append(xx)
for iy in range(npatchy):
  yy = opatchy + (iy*dpatchy)
  listy.append(yy)

# define some file names
infile = 'sh-ph2-down16-supercube-master.H'
headerfile = 'hdr-master.h'
windowfile = 'window-master.h'
sortfile = 'sort-master.h'
outfile = 'sh-ph2-down16-supergather-patch-master-%skm.H' %(size)

# calculate n2 (number of correlations total)
ncorr_com = 'Get < %s parform=n n2' %(infile)
ncorr = commands.getoutput(ncorr_com)
print ncorr

# convert units in the header
header(infile,headerfile,run)

# loop over the x and y directions for patches
catxfiles = []
for y in listy:

  stackfiles = []
  for x in listx:

    # window out appropriate stations
    windowkey(headerfile,windowfile,x,y,lpatchx,lpatchy,run)

    # check if anything is returned (else fill with zeros)
    get_com = 'Get < %s@@ parform=n n2' %(windowfile)
    nstations = commands.getoutput(get_com)

    # if nothing is found
    if (nstations == ncorr):
      # save filename to list and fill with zeros
      stackfile = 'stack-master-%s-%s.h' %(x,y)
      stackfiles.append(stackfile)
      # grab parameters to make file of appropriate size
      get_com = 'Get < %s parform=n n1' %(windowfile)
      nt = commands.getoutput(get_com)
      get_com = 'Get < %s parform=n o1' %(windowfile)
      ot = commands.getoutput(get_com)
      get_com = 'Get < %s parform=n d1' %(windowfile)
      dt = commands.getoutput(get_com)
      fillzeros(stackfile,nt,ot,dt,lpatchx,lpatchy,lh,run)

    # if stations exist
    else:
      # sort everything by radial offset and azimuth
      sort(windowfile,sortfile,lpatchx,lpatchy,lh,run)
     
      # save filename to list and stack over everything
      stackfile = 'stack-master-%s-%s.h' %(x,y)
      stackfiles.append(stackfile)
      stack(sortfile,stackfile,run)

  # with all the files for one y-coordinate and all x-coordinates, cat them
  # output: n1: time, n2: offset, n3: number of x-coordinates
  catxfile = 'cat-master-%s.h' %(y)
  catxfiles.append(catxfile)
  catx(stackfiles,catxfile,run)

# should have all y-coordinate files by now, so just need to cat
caty(catxfiles,outfile,run)

# echo the correct information
echo(outfile,run)
